@@ -20,44 +20,47 @@ Options:
 import os
 import json
 import time
+import pytz
 
 from docopt import docopt
 from typing import Optional
-from datetime import datetime, timedelta
+from datetime import date, datetime, timedelta
 from loguru import logger
+from tqdm import tqdm
 
 from fsm_pull import __version__
 from fsm_pull.connection import Connection
+from fsm_pull.protobuf.records_pb2 import Call, Turn, Utterance
 
 
-def convert_date(date_string: str) -> str:
+def convert_date(date_string: str, timezone="Asia/Kolkata") -> str:
+    if not isinstance(date_string, str):
+        raise ValueError("Date should be ddmmyyyy format")
+
+    if date_string != "now" and len(date_string) != 8:
+        raise ValueError("Date should be ddmmyyyy format")
+
+    current_time = datetime.now() + timedelta(days=1)
+    current_time = current_time.replace(tzinfo=pytz.timezone(timezone))
+    start_limit = datetime(2016, 1, 6).replace(tzinfo=pytz.timezone(timezone))
 
     if date_string == "now":
-        return (datetime.now() + timedelta(days=1)).strftime(
-            "%Y-%m-%d"
-        ) + "T00:00:00+05:30"
+        return current_time.isoformat()
+    else:
+        given_date = datetime.strptime(date_string, "%d%m%Y")
+        given_date = given_date.replace(tzinfo=pytz.timezone(timezone))
 
-    try:
-
-        if len(date_string) == 8:
-            given_date = datetime.strptime(date_string, "%d%m%Y")
-            if (
-                datetime.strptime("01062016", "%d%m%Y")
-                <= given_date
-                <= (datetime.now() + timedelta(days=1))
-            ):
-                return given_date.strftime("%Y-%m-%d") + "T00:00:00+05:30"
-
-        raise ValueError
-
-    except ValueError:
-        raise ValueError(
-            "Please check date format and if date is in range,"
-            "If date is 5th August 1947, Please enter 05081947(DDMMYYYY)"
-        )
+        if start_limit <= given_date <= current_time:
+            return given_date.isoformat()
+        else:
+            raise ValueError(
+                f"Date {date_string} is not in the range of {start_limit} to {current_time}"
+            )
 
 
 def parse_metadata_json(conversations):
+    if not conversations:
+        return []
     for conversation in conversations:
         if "metadata" in conversation:
             conversation["metadata"] = json.loads(conversation["metadata"])
@@ -71,6 +74,8 @@ def get_call_list(
     end: str = "now",
     call_quantity: int = 0,
     flow_uuid: Optional[str] = None,
+    custom_search_key: Optional[str] = None,
+    custom_search_value: Optional[str] = None,
 ):
     start = convert_date(start)
     end = convert_date(end)
@@ -83,9 +88,17 @@ def get_call_list(
 
     call_list = []
     turn_count = 0
-    calls = new_connection.fetch_calls(lang_code, start, end, call_quantity, flow_uuid=flow_uuid)
+    calls = new_connection.fetch_calls(
+        lang_code,
+        start,
+        end,
+        call_quantity,
+        flow_uuid=flow_uuid,
+        custom_search_key=custom_search_key,
+        custom_search_value=custom_search_value,
+    )
 
-    for call in calls:
+    for call in tqdm(calls):
         turns = new_connection.fetch_turns(call["uuid"])
         turns["conversations"] = parse_metadata_json(turns["conversations"])
 
@@ -97,7 +110,9 @@ def get_call_list(
 
         call_list.append(call)
 
-    logger.info(f"pulled {len(calls)} calls ({turn_count} turns) in {time.time() - proc_start_time:.2f}s")
+    logger.info(
+        f"pulled {len(calls)} calls ({turn_count} turns) in {time.time() - proc_start_time:.2f}s"
+    )
     return call_list
 
 
